import java.util.*;
import java.io.*;

/**
 * Skeleton of a class to represent a finite automaton in both
 * non-determinstic and determinstic variants - fill in the blanks (...)
 */
public class Automaton {
	// Automaton consists of states and labels, transitions are managed by states
	private Set states, labels;
	private State currentState;
	// Utility data structure to map unique names to corresponding instance
	private Map nameToState, nameToLabel;
	...

	// Teilaufgabe a) ---------------------------------------------------------

	public Automaton() {
		states = new HashSet(); labels = new HashSet();
		nameToState = new HashMap(); nameToLabel = new HashMap();
		...
	}

	// Add a new state to the automaton, possibly as start or terminal state
	public void addState(String name, boolean isStart, boolean isTerminal) {
		...
	}

	// Add a new transition to the automaton between possibly existing states
	public void addTransition(String from, String to, String label) {
		...
	}

        // Retrieve current state of the automaton
        public State getCurrentState() {
                return currentState;
        }

        // Set current state of the automaton
        public void setCurrentState(State s) {
                currentState = s;
        }

	// Initialize the automaton, called before execution
	public void init() {
		initTransitionMatrix();
		...
	}

	/**
         *  Execute a single step of the automaton by providing a language symbol,
         *  not changing its internal state (call setCurrentState() later to do so)
	 *  Returns a set of states which is either
	 *    null, if no valid symbol has been provided,
	 *    empty, if no valid transition exists (i.e. the word is not accepted),
	 *    a single state, if the automaton is deterministic for given symbol,
	 *    a number of states, if the automaton is nondeterministic for given symbol.
	 * */
	public Set step(String symbol) {
		...
	}

	// Teilaufgabe b) ---------------------------------------------------------

	// Construct automaton from given regular grammar with productions of form
	// <nonterminal> -> <epsilon>|terminal|terminal <nonterminal>
	public void readGrammar(String fileName) {
		...
	}

	// Detect a number of special tokens
	private boolean isEpsilon(String token) {
		if (token.equals("<epsilon>"))
			return true;
		return false;
	}

	private boolean isStart(String token) {
		if (token.equals("<S>"))
			return true;
		return false;
	}

	// Formatted output of automaton
	public String toString() {
		String result = new String();
		result += "States: ";
		Iterator sit = states.iterator();
		while (sit.hasNext()) {
			result += (State) sit.next();
			if (sit.hasNext()) result += ", ";
		}
		result += "\nLabels: ";
		Iterator lit = labels.iterator();
		while (lit.hasNext()) {
			result += (Label) lit.next();
			if (lit.hasNext()) result += ", ";
		}
		result += "\nTransitions:\n";
		sit = states.iterator();
		while (sit.hasNext()) {
			State s = (State) sit.next();
			Iterator tit = s.getOutgoingTransitions().iterator();
			while (tit.hasNext()) {
				Transition t = (Transition) tit.next();
				result += "\t" + s + " -" + t + "-> " + t.getTarget() +"\n";
			}
		}
		result += "\n";
		return result;
	}
}
