import java.util.*;

/**
 * Used to test functionality of class Automaton
 *
 */
public class Test {

	public static void main(String[] args) {
		Test t = new Test();
		t.testDFA();
		System.out.println("*************");
		t.testGrammar();
	}

	// Deterministic finite automaton for regular language L = ab(ab)*b
	private void testDFA() {
		Automaton a = new Automaton();
		a.addState("<S>", true, false);
		a.addState("<S1>", false, false);
		a.addState("<S2>", false, false);
		a.addState("<F>", false, true);
		a.addTransition("<S>", "<S1>", "a");
		a.addTransition("<S1>", "<S2>", "b");
		a.addTransition("<S2>", "<S1>", "a");
		a.addTransition("<S2>", "<F>", "b");
		a.init();
                System.out.print(a);
		accept(a, "a b");       // not in L
                System.out.println("-----");
		a.init();
		accept(a, "a b b");     // in L
                System.out.println("-----");
		a.init();
		accept(a, "a b a b");   // not in L
                System.out.println("-----");
		a.init();
		accept(a, "a b a b b"); // in L
	}

	// Provided grammar defines regular language L = a(a|b)*
	private void testGrammar() {
		Automaton a = new Automaton();
		a.readGrammar("grammar.txt");
		a.init();
		System.out.print(a);
		accept(a, "b a b"); // not in L
                System.out.println("-----");
		a.init();
		accept(a, "a a b"); // in L
	}

	/** Test acceptance of word with space-separated symbols
	 *  Note that a nondeterministic automaton may not always accept a given
	 *  word of the language, at least one accepting run is sufficient!
	 */
	private void accept(Automaton a, String word) {
		Set nextStates = null;
		StringTokenizer tokenizer = new StringTokenizer(word);
                // Iterate steps for all symbols of given word
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			System.out.print(a.getCurrentState() + " -" + token + "-> ");
			// Determine set of reachable states for given symbol
                        nextStates = a.step(token);
			Iterator sit = nextStates.iterator();
			while (sit.hasNext()) {
				System.out.print(sit.next());
				if (sit.hasNext()) System.out.print(",");
			}
			System.out.print("\n");
                        // Determine next state of the automaton if possible
                        if (nextStates == null || nextStates.isEmpty()) {
                                System.out.println("Automaton did not accept word \"" + word + "\"");
                                return;
                        } else {
                              if (nextStates.size() == 1) {
                                  // Choose next state deterministically
                                  a.setCurrentState((State) nextStates.iterator().next());
                              } else {
                                  // Choose arbitrary next state, possibly avoiding final state
                                  State sn = null;
                                  sit = nextStates.iterator();
                                  while (sit.hasNext()) {
                                      sn = (State) sit.next();
                                      if (!sn.isTerminal()) break;
                                  }
                                  a.setCurrentState(sn);
                              }
                        }
		}
                // Check last set of states for inclusion of a terminal state
                Iterator sit = nextStates.iterator();
                while (sit.hasNext()) {
                        State s = (State) sit.next();
                        if (s.isTerminal()) {
                                System.out.println("Automaton accepted word \"" + word + "\"");
                                return;
                        }
                }
                System.out.println("Automaton did not accept word \"" + word + "\"");
	}
}
